# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import object

from monolithe.lib import SDKUtils


class SpecificationAttribute(object):
    """ Define an attribute of an object

    """
    def __init__(self, specification=None, data=None):
        """ Define an attribute

            Example:
                name: associatedGatewayID
                local_name: associated_gateway_id
                local_type: str
        """
        # Main attributes
        self.description = None
        self.local_name = None
        self.local_type = None

        # Other attributes
        self._name = None
        self.channel = None
        self.allowed_chars = None
        self.allowed_choices = None
        self.autogenerated = False
        self.availability = None
        self.creation_only = False
        self.default_order = False
        self.default_value = None
        self.deprecated = False
        self.filterable = False
        self.format = "free"
        self.max_length = None
        self.max_value = None
        self.min_length = None
        self.min_value = None
        self.orderable = False
        self.read_only = False
        self.required = False
        self.unique = False
        self.unique_scope = None
        self._type = None
        self.exposed = False
        self.transient = False
        self.subtype = None
        self.userlabel = None

        self.specification = specification
        # Load information from data
        if data:
            self.from_dict(data)

    @property
    def type(self):
        """
        """
        return self._type

    @type.setter
    def type(self, value):
        """
        """
        language = self.specification.monolithe_config.language if self.specification and self.specification.monolithe_config else 'python'

        self._type = SDKUtils.massage_type_name(type_name=value)
        self.local_type = SDKUtils.get_type_name_in_language(type_name=value, sub_type=self.subtype, language=language)

    @property
    def name(self):
        """
        """
        return self._name

    @name.setter
    def name(self, value):
        """
        """
        self._name = value

        language = self.specification.monolithe_config.language if self.specification and self.specification.monolithe_config else 'python'

        if self.specification and self.specification.monolithe_config:
            self.local_name = SDKUtils.get_idiomatic_name_in_language(name=self.specification.monolithe_config.map_attribute(self.specification.rest_name, value), language=language)
        else:
            self.local_name = SDKUtils.get_idiomatic_name_in_language(name=value, language=language)

    def from_dict(self, data):
        """

        """
        try:
            # this must be done before setting the type
            self.subtype = data["subtype"] if "subtype" in data else None

            # mandatory characteristics
            self.name = data["name"]
            self.description = data["description"]
            self.type = data["type"]

            # optional characteristics
            self.allowed_chars = data["allowed_chars"] if "allowed_chars" in data else None
            self.allowed_choices = data["allowed_choices"] if "allowed_choices" in data else None
            self.autogenerated = data["autogenerated"] if "autogenerated" in data else False
            self.channel = data["channel"] if "channel" in data else None
            self.creation_only = data["creation_only"] if "creation_only" in data else False
            self.default_order = data["default_order"] if "default_order" in data else False
            self.default_value = data["default_value"] if "default_value" in data else None
            self.deprecated = data["deprecated"] if "deprecated" in data else False
            self.exposed = data["exposed"] if "exposed" in data else False
            self.filterable = data["filterable"] if "filterable" in data else False
            self.format = data["format"] if "format" in data else "free"
            self.max_length = data["max_length"] if "max_length" in data else None
            self.max_value = data["max_value"] if "max_value" in data else None
            self.min_length = data["min_length"] if "min_length" in data else None
            self.min_value = data["min_value"] if "min_value" in data else None
            self.orderable = data["orderable"] if "orderable" in data else False
            self.read_only = data["read_only"] if "read_only" in data else False
            self.required = data["required"] if "required" in data else False
            self.transient = data["transient"] if "transient" in data else False
            self.unique = data["unique"] if "unique" in data else False
            self.unique_scope = data["unique_scope"] if "unique_scope" in data else None
            self.userlabel = data["userlabel"] if "userlabel" in data else None

        except Exception as ex:
            raise Exception("Unable to parse attribute %s for specification %s: %s" % (self.name, self.specification.rest_name, ex))

    def to_dict(self):
        """ Transform an attribute to a dict
        """
        data = {}

        # mandatory characteristics
        data["name"] = self.name
        data["description"] = self.description if self.description and len(self.description) else None
        data["type"] = self.type if self.type and len(self.type) else None
        data["allowed_chars"] = self.allowed_chars if self.allowed_chars and len(self.allowed_chars) else None
        data["allowed_choices"] = self.allowed_choices
        data["autogenerated"] = self.autogenerated
        data["channel"] = self.channel if self.channel and len(self.channel) else None
        data["creation_only"] = self.creation_only
        data["default_order"] = self.default_order
        data["default_value"] = self.default_value if self.default_value and len(self.default_value) else None
        data["deprecated"] = self.deprecated
        data["exposed"] = self.exposed
        data["filterable"] = self.filterable
        data["format"] = self.format if self.format and len(self.format) else None
        data["max_length"] = int(self.max_length) if self.max_length is not None else None
        data["max_value"] = int(self.max_value) if self.max_value is not None else None
        data["min_length"] = int(self.min_length) if self.min_length is not None else None
        data["min_value"] = int(self.min_value) if self.min_value is not None else None
        data["orderable"] = self.orderable
        data["read_only"] = self.read_only
        data["required"] = self.required
        data["transient"] = self.transient
        data["unique"] = self.unique
        data["uniqueScope"] = self.unique_scope if self.unique_scope and len(self.unique_scope) else None
        data["subtype"] = self.subtype if self.subtype and len(self.subtype) else None
        data["userlabel"] = self.userlabel if self.userlabel and len(self.userlabel) else None

        return data



from monolithe.generators.lib import TemplateFileWriter
from monolithe.specifications import SpecificationAttribute
from monolithe.lib import Printer
import os
import shutil
import json


class APIVersionWriter(TemplateFileWriter):
    """ This class is reponsible to write files for a particular api version. """

    def __init__(self, monolithe_config, api_info):
        
        super(APIVersionWriter, self).__init__(package="monolithe.generators.lang.javascript")

        output = monolithe_config.get_option("output", "transformer")
        
        self.locale_on = monolithe_config.get_option("locale", "transformer", fallback=True)
        self.model_directory = "%s/javascript/%s/models" % (output, api_info["version"])
        self.abstract_directory =  "%s/abstract" % self.model_directory
        self.enum_directory =  "%s/enums" % self.model_directory
        self.locale_directory = "%s/javascript/%s/locales/en" % (output, api_info["version"])

        if os.path.exists(self.model_directory):
            shutil.rmtree(self.model_directory)

        if os.path.exists(self.locale_directory):
            shutil.rmtree(self.locale_directory)

        self.api_root = api_info["root"]
        self._class_prefix = monolithe_config.get_option("class_prefix", "transformer")
        
        self._read_config()


    def _read_config(self):
        """ This method reads provided json config file.
        """
        
        this_dir = os.path.dirname(__file__)        
        config_file = os.path.abspath(os.path.join(this_dir, "..", "config", "config.json"))

        self.generic_enum_attrs = []
        self.base_attrs = []
        self.generic_enums = []
        self.named_entity_attrs = []
        self.overide_generic_enums = []
        self.enum_attrs_for_locale = {}
        self.generic_enum_attrs_for_locale = {}
        self.list_subtypes_generic = []
        
        Printer.log("Configuration file: %s" % (config_file))

        if (os.path.isfile(config_file)):
            with open(config_file, 'r') as input_json:
                json_config_data = json.load(input_json) 
                
            self.base_attrs = json_config_data['base_attrs']
            self.generic_enums =  json_config_data['generic_enums']
            self.named_entity_attrs = json_config_data['named_entity_attrs']
            self.overide_generic_enums = json_config_data['overide_generic_enums']
            self.list_subtypes_generic = json_config_data['list_subtypes_generic']

            for enum_name, values in self.generic_enums.iteritems():
                enum_attr =  SpecificationAttribute()
                enum_attr.name = enum_name
                enum_attr.allowed_choices = values
                self.generic_enum_attrs.append(enum_attr)
        else:
            Printer.log("Configuration file missing: %s" % (config_file))
        
        
    def perform(self, specifications):
        """ This method is the entry point of javascript code writer. Monolithe will call it when
        the javascript plugin is to generate code.
        """
        self.enum_list = []
        self.model_list = []
        self.job_commands = filter(lambda attr: attr.name == 'command', specifications.get("job").attributes)[0].allowed_choices
        #Printer.log("job_commands: %s" % (self.job_commands))
            
        self._write_abstract_named_entity()
        
        self.entity_names = [specification.entity_name for rest_name, specification in specifications.iteritems()]

        for rest_name, specification in specifications.iteritems():
            self._write_model(specification=specification)

        #self._write_generic_enums()
        
        self.write(destination = self.model_directory,
            filename="index.js",
            template_name="model_index.js.tpl",
            class_prefix = self._class_prefix,
            model_list = sorted(self.model_list))
            
        self.write(destination = self.enum_directory,
                    filename="index.js",
                    template_name="enum_index.js.tpl",
                    class_prefix = self._class_prefix,
                    enum_list = sorted(self.enum_list))
                    
        self._write_locales(specifications)            

    def _write_locales(self, specifications):
        if self.locale_on:
            self.locales_list = []
            for rest_name, specification in specifications.items():
                self._format_description_text(specification)            
                filename = "%s.json" % (rest_name)
                self.locales_list.append(rest_name)
                self.write(destination = self.locale_directory,
                    filename=filename,
                    template_name="locale_entity.json.tpl",
                    specification = specification,
                    enum_attrs = {})
                    
            self.write(destination = self.locale_directory,
                filename="index.js",
                template_name="locales_index.js.tpl",
                locales_list = sorted(self.locales_list))
            
    def _format_description_text(self, specification):
        if specification.description:
            specification.description = specification.description.replace('"', "'")  
            
        for attribute in specification.attributes:
            if attribute.description:
                attribute.description = attribute.description.replace('"', "'")

    def _write_abstract_named_entity(self):
        """ This method generates AbstractNamedEntity class js file.
        """
        filename = "%sAbstractNamedEntity.js" % (self._class_prefix)
                
        superclass_name = "%sEntity" % (self._class_prefix)
        
        # write will write a file using a template.
        # mandatory params: destination directory, destination file name, template file name
        # optional params: whatever that is needed from inside the Jinja template

        self.write(destination = self.abstract_directory,
                    filename = filename,
                    template_name = "abstract_named_entity.js.tpl",
                    class_prefix = self._class_prefix,
                    superclass_name = superclass_name)
                    
    def _write_model(self, specification):
        """ This method writes the ouput for a particular specification.
        """
        
        if specification.allowed_job_commands and not (set(specification.allowed_job_commands).issubset(self.job_commands)):
            raise Exception("Invalid allowed_job_commands %s specified in entity %s" % (specification.allowed_job_commands, specification.entity_name))
            
        specification.supportsAlarms = len(filter(lambda child_api : child_api.rest_name == "alarm", specification.child_apis)) == 1
        
        specification.supportsPermissions = len(filter(lambda child_api : child_api.rest_name == "enterprisepermission" or child_api.rest_name == "permission", specification.child_apis)) > 0

        specification.supportsDeploymentFailures = len(filter(lambda child_api : child_api.rest_name == "deploymentfailure", specification.child_apis)) == 1

        filename = "%s%s.js" % (self._class_prefix, specification.entity_name)

        self.model_list.append("%s%s" %(self._class_prefix, specification.entity_name))
        
        isNamedEntity = self._isNamedEntity(attributes=specification.attributes) if specification.rest_name else False
                
        superclass_name = "RootEntity" if specification.rest_name == self.api_root else "AbstractNamedEntity" if isNamedEntity  else "AbstractModel" if not specification.rest_name else "Entity"
        # write will write a file using a template.
        # mandatory params: destination directory, destination file name, template file name
        # optional params: whatever that is needed from inside the Jinja template

        specification.attributes_modified = [attribute for attribute in specification.attributes if (attribute.name not in self.base_attrs and (not isNamedEntity or attribute.name not in self.named_entity_attrs))]

        enum_attributes=[attribute for attribute in specification.attributes_modified if attribute.allowed_choices]
                
        enum_attrs_to_import = enum_attributes[:]
        generic_enum_attrs_in_entity = {}
        generic_enum_attributes_to_import = []

        for attr in enum_attributes:
            if attr.local_type == "list" and attr.subtype == "enum" and attr.default_value:
                attr.default_value = attr.default_value.translate({ord(i): None for i in ' []"'}).split(",")
                if not all(defval in attr.allowed_choices for defval in attr.default_value):
                    raise Exception("Invalid default value specified for attribute %s in entity %s" % (attr.name, specification.entity_name))
                                        
            if specification.rest_name in self.overide_generic_enums and  attr.name in self.overide_generic_enums[specification.rest_name]:
                continue
            for generic_enum_attr in self.generic_enum_attrs:
                if set(attr.allowed_choices) & set(generic_enum_attr.allowed_choices):
                    generic_enum_attrs_in_entity[attr.name] = generic_enum_attr
                    enum_attrs_to_import.remove(attr)
                    generic_enum_attributes_to_import.append(generic_enum_attr.name)

        
        self._write_enums(entity_name=specification.entity_name, attributes=enum_attrs_to_import)

        self.generic_enum_attrs_for_locale[specification.entity_name] = generic_enum_attrs_in_entity.values()
        
        object_subtypes = set([attribute.subtype for attribute in specification.attributes if (attribute.local_type == "object"  and attribute.subtype)])

        invalid_object_attributes=[attribute.name for attribute in specification.attributes_modified if (attribute.local_type == "object" and not attribute.subtype in self.entity_names)]

        if invalid_object_attributes:
            Printer.log("Spec: %s: Attributes %s use invalid subtypes %s" % (filename, invalid_object_attributes, object_subtypes))

        list_subtypes = set([attribute.subtype for attribute in specification.attributes if (attribute.local_type == "list" and attribute.subtype not in self.list_subtypes_generic)])

        invalid_list_attributes=[attribute.name for attribute in specification.attributes_modified if (attribute.local_type == "list" and not attribute.subtype in self.entity_names and not attribute.subtype in self.list_subtypes_generic)]

        if invalid_list_attributes:
            Printer.log("Spec: %s: Attributes %s use invalid list subtypes %s" % (filename, invalid_list_attributes, list_subtypes))

        if 'object' in list_subtypes:
            list_subtypes.remove('object')
        if 'entity' in list_subtypes:
            list_subtypes.remove('entity')

        self.write(destination = self.model_directory,
                    filename = filename,
                    template_name = "entity.js.tpl",
                    class_prefix = self._class_prefix,
                    specification = specification,
                    superclass_name = superclass_name,
                    enum_attrs_to_import = enum_attrs_to_import,
                    generic_enum_attributes = generic_enum_attrs_in_entity,
                    generic_enum_attributes_to_import = set(generic_enum_attributes_to_import),
                    subtypes_for_import = object_subtypes.union(list_subtypes))

    def _isNamedEntity(self, attributes):
        attr_names = [attr.name for attr in attributes]
        named_attrs_applicable = len(self.named_entity_attrs) > 0 and set(self.named_entity_attrs).issubset(attr_names)
        if (named_attrs_applicable):
            name_attr = filter(lambda attr: attr.name == 'name', attributes)[0]
            if ((not name_attr.required) or (not name_attr.filterable)):
                return False
            desc_attr = filter(lambda attr: attr.name == 'description', attributes)[0]
            if (desc_attr.required or (not desc_attr.filterable)):
                return False
            return True
        return False
    
    def _write_enums(self, entity_name, attributes):
        """ This method writes the ouput for a particular specification.
        """
        
        self.enum_attrs_for_locale[entity_name] = attributes;

        for attribute in attributes:
            enum_name = "%s%sEnum" % (entity_name, attribute.name[0].upper() + attribute.name[1:])
            self.enum_list.append(enum_name)
            filename = "%s%s.js" % (self._class_prefix, enum_name)
            self.write(destination = self.enum_directory,
                        filename=filename,
                        template_name="enum.js.tpl",
                        class_prefix = self._class_prefix,
                        enum_name = enum_name,
                        allowed_choices = set(attribute.allowed_choices))

    def _write_generic_enums(self):
        """ This method generates generic enum classes.
        """

        self._write_enums(entity_name='', attributes=self.generic_enum_attrs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import print_function
import argparse
import sys

from monolithe.specifications.directory_manager import FolderManager, RepositoryManager
from monolithe.generators import SDKGenerator


def main(argv=sys.argv):
    """

    """
    parser = argparse.ArgumentParser(description="Generates a SDK according from a specification set")

    parser.add_argument("-b", "--branches",
                        dest="branches",
                        metavar="branches",
                        help="The branches of the specifications to use to generate the documentation (examples: \"master 3.2\")",
                        nargs="*",
                        type=str)

    parser.add_argument("-f", "--folder",
                        dest="folder",
                        metavar="folder",
                        required=True,
                        help="Path of the specifications folder. If set, all other attributes will be ignored",
                        type=str)

    parser.add_argument("-c", "--config",
                        dest="config_path",
                        metavar="config_path",
                        help="Path the monolithe configuration file",
                        type=str)

    parser.add_argument("--vanilla-prefix",
                        dest="vanilla_prefix",
                        help="Prefix added to all vanilla path declared in the monolithe configuration file",
                        required=False,
                        type=str)

    parser.add_argument("--generation-version",
                        dest="generation_version",
                        help="Overwrite the sdk version given in monolithe.conf",
                        required=False,
                        type=str)

    parser.add_argument("-L", "--language",
                        dest="language",
                        help="Choose the output language of the SDK. Default is python",
                        default='python',
                        type=str)

    args = parser.parse_args()

    if args.branches:
        directory_manager = RepositoryManager(
            args.folder, config_path=args.config_path)
    else:
        directory_manager = FolderManager(
            args.folder, config_path=args.config_path)

    monolithe_config = directory_manager.monolithe_config

    if monolithe_config and args.vanilla_prefix:
        monolithe_config.set_option("user_vanilla", "%s/%s" % (args.vanilla_prefix, monolithe_config.get_option("user_vanilla", "transformer")), "transformer")

    if monolithe_config and args.generation_version:
        monolithe_config.set_option("version", args.generation_version, "transformer")

    if monolithe_config:
        monolithe_config.language = args.language

    generator = SDKGenerator(directory_manager, args.branches)
    generator.run(args.branches)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import str
from builtins import map
from builtins import object

import re
from unittest2 import TestSuite
from .testcase import CourgetteTestCase


class _TestMaker(object):
    """ Make tests
    """

    IGNORED_ATTRIBUTES = ["id", "parent_id", "parent_type", "creation_date", "owner", "last_updated_date", "last_updated_by", "external_id"]

    def __init__(self, helper):
        """ Initializes a TestMaker

        """
        self._object_registry = dict()
        self._attributes_registry = dict()
        self.helper = helper

    def register_test(self, function_name, **conditions):
        """ Register test for all attributes

        """
        if function_name not in self._object_registry:
            self._object_registry[function_name] = conditions

    def register_test_for_attribute(self, function_name, **conditions):
        """ Register an attribute test for all given conditions

        """
        if function_name not in self._attributes_registry:
            self._attributes_registry[function_name] = conditions

    def does_attribute_meet_condition(self, attribute, conditions):
        """ Check if the attribute meet all the given conditions

            Args:
                attribute: the attribute information
                conditions: a dictionary of condition to match

            Returns:
                True if the attribute match all conditions. False otherwise
        """
        if conditions is None or len(conditions) == 0:
            return True

        for attribute_name, attribute_value in conditions.items():
            value = getattr(attribute, attribute_name, False)
            if value != attribute_value and bool(value) != attribute_value:
                return False

        return True

    def make_tests(self, sdkobject, testcase):
        """ Make all tests that should be run for the given object in the specified testcase

            Args:
                sdkobject: the sdk object
                testcase: the test case

            Returns:
                It returns a dictionary of all tests to run

        """
        tests = dict()
        attributes = sdkobject.get_attributes()

        for attribute in attributes:

            if attribute.local_name in self.IGNORED_ATTRIBUTES:
                continue

            for function_name, conditions in self._attributes_registry.items():
                if self.does_attribute_meet_condition(attribute, conditions):
                    (test_name, test_func) = self._create_test(testcase=testcase, sdkobject=sdkobject, function_name=function_name, attribute=attribute)
                    tests[test_name] = test_func

        for function_name, infos in self._object_registry.items():
            (test_name, test_func) = self._create_test(testcase=testcase, sdkobject=sdkobject, function_name=function_name)
            tests[test_name] = test_func

        return tests

    def _create_test(self, testcase, function_name, sdkobject, attribute=None):
        """ Create a test method for the sdkoject

            Args:
                testcase: the testcase to that should manage the method
                function_name: the name of the method in the testcase
                sdkobject: the object that should be tested
                attribute: the attribute information if necessary

            Returns:
                It returns a tuple (name, method) that represents the test method

        """
        func = getattr(testcase, function_name)
        object_name = sdkobject.rest_name

        # Name that will be displayed
        test_name = ""
        rep = dict()
        rep["object"] = object_name

        if attribute:
            rep["attribute"] = attribute.local_name

        rep = dict((re.escape(k), v) for k, v in rep.items())
        pattern = re.compile("|".join(list(rep.keys())))

        if function_name.startswith("_"):
            function_name = function_name[1:]

        test_name = pattern.sub(lambda m: rep[re.escape(m.group(0))], function_name)

        # Prepare and add test method to test suite
        test_func = None

        if attribute:
            test_func = lambda self, attribute=attribute: func(self, attribute)
        else:
            test_func = lambda self: func(self)

        test_func.__name__ = str(test_name)

        return (test_name, test_func)


# CREATE TESTS

class CreateTestMaker(_TestMaker):
    """ TestCase for create objects

    """
    def __init__(self, parent, sdkobject, helper):
        """ Initializes a test case for creating objects

        """
        super(CreateTestMaker, self).__init__(helper=helper)
        self.parent = parent
        self.sdkobject = sdkobject

        # Object tests
        self.register_test("_test_create_object_with_all_valid_attributes_should_succeed")
        self.register_test("_test_create_object_without_authentication_should_fail")

        # Attribute tests
        self.register_test_for_attribute("_test_create_object_with_required_attribute_as_none_should_fail", is_required=True)
        self.register_test_for_attribute("_test_create_object_with_attribute_not_in_allowed_choices_list_should_fail", has_choices=True)
        self.register_test_for_attribute("_test_create_object_with_attribute_as_none_should_succeed", is_required=False)

    def suite(self):
        """ Inject generated tests

        """
        CreateTestCase.parent = self.parent
        CreateTestCase.sdkobject = self.sdkobject
        CreateTestCase.helper = self.helper

        tests = self.make_tests(sdkobject=self.sdkobject, testcase=CreateTestCase)
        for test_name, test_func in tests.items():
            setattr(CreateTestCase, test_name, test_func)

        return TestSuite(list(map(CreateTestCase, tests)))


class CreateTestCase(CourgetteTestCase):

    def __init__(self, methodName="runTest"):
        """ Initialize

        """
        CourgetteTestCase.__init__(self, methodName=methodName)
        self.pristine_sdkobject = self.sdkobject.copy()

    def setUp(self):
        """ Setting up create test

        """
        self.last_connection = None
        self.sdkobject = self.pristine_sdkobject.copy()

    def tearDown(self):
        """ Clean up environment

        """
        if self.sdkobject and self.sdkobject.id:
            self.sdkobject.delete()
            self.sdkobject.id = None

    # Objects tests
    def _test_create_object_without_authentication_should_fail(self):
        """ Create an object without authentication """

        self.helper.set_api_key(None)
        (obj, connection) = self.parent.create_child(self.sdkobject)
        self.last_connection = connection
        self.helper.set_api_key(self.helper.session.root_object.api_key)

        self.assertConnectionStatus(connection, 401)

    def _test_create_object_with_all_valid_attributes_should_succeed(self):
        """ Create an object with all its valid attributes should always succeed with 201 response

        """
        (obj, connection) = self.parent.create_child(self.sdkobject)
        self.last_connection = connection

        self.assertConnectionStatus(connection, 201)
        self.assertEquals(obj.to_dict(), self.sdkobject.to_dict())

    # Attributes tests
    def _test_create_object_with_required_attribute_as_none_should_fail(self, attribute):
        """ Create an object with a required attribute as None """

        setattr(self.sdkobject, attribute.local_name, None)
        (obj, connection) = self.parent.create_child(self.sdkobject)
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertErrorEqual(connection.response.errors, title="Invalid input", description="This value is mandatory.", rest_name=attribute.rest_name)

    def _test_create_object_with_attribute_as_none_should_succeed(self, attribute):
        """ Create an objet with an attribute as none """

        setattr(self.sdkobject, attribute.local_name, None)
        (obj, connection) = self.parent.create_child(self.sdkobject)
        self.last_connection = connection

        self.assertConnectionStatus(connection, 201)
        # self.assertIsNone(getattr(obj, attribute.local_name), "%s should be none but was %s instead" % (attribute.local_name, getattr(obj, attribute.local_name)))

    def _test_create_object_with_attribute_not_in_allowed_choices_list_should_fail(self, attribute):
        """ Create an object with a wrong choice attribute """

        setattr(self.sdkobject, attribute.local_name, "A random value")
        (obj, connection) = self.parent.create_child(self.sdkobject)
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertErrorEqual(connection.response.errors, title="Invalid input", description="Invalid input", rest_name=attribute.rest_name)


# UPDATE TESTS

class UpdateTestMaker(_TestMaker):
    """ TestCase for updating objects
    """

    def __init__(self, parent, sdkobject, helper):
        """ Initializes a test case for updating objects
        """
        super(UpdateTestMaker, self).__init__(helper=helper)
        self.parent = parent
        self.sdkobject = sdkobject

        # Object tests
        # self.register_test("_test_update_object_with_same_attributes_should_fail") # this feature is completely stupid anyway
        self.register_test("_test_update_object_without_authentication_should_fail")

        # Attribute tests
        self.register_test_for_attribute("_test_update_object_with_attribute_not_in_allowed_choices_list_should_fail", has_choices=True)
        self.register_test_for_attribute("_test_update_object_with_required_attribute_as_none_should_fail", is_required=True)
        self.register_test_for_attribute("_test_update_object_with_attribute_as_none_should_succeed", is_required=False)
        # self.register_test_for_attribute("_test_update_object_with_attribute_with_choices_as_none_should_fail", has_choices=True)

    def suite(self):
        """ Inject generated tests
        """
        UpdateTestCase.parent = self.parent
        UpdateTestCase.sdkobject = self.sdkobject
        UpdateTestCase.helper = self.helper

        tests = self.make_tests(sdkobject=self.sdkobject, testcase=UpdateTestCase)
        for test_name, test_func in tests.items():
            setattr(UpdateTestCase, test_name, test_func)

        return TestSuite(list(map(UpdateTestCase, tests)))


class UpdateTestCase(CourgetteTestCase):

    def __init__(self, methodName="runTest"):
        """ Initialize
        """
        CourgetteTestCase.__init__(self, methodName=methodName)
        self.pristine_sdkobject = self.sdkobject.copy()

    def setUp(self):
        """ Setting up create test
        """
        self.last_connection = None
        self.sdkobject = self.pristine_sdkobject.copy()
        self.parent.create_child(self.sdkobject)

    def tearDown(self):
        """ Clean up environment
        """
        self.sdkobject.delete()

    # Objects tests
    def _test_update_object_without_authentication_should_fail(self):
        """ Update an object without authentication
        """
        self.helper.set_api_key(None)
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection
        self.helper.set_api_key(self.helper.session.root_object.api_key)

        self.assertConnectionStatus(connection, 401)

    def _test_update_object_with_same_attributes_should_fail(self):
        """ Update an object with same attributes should always fail with 409 error
        """
        (obj, connection) = self.sdkobject.save()
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertErrorEqual(connection.response.errors, title="No changes to modify the entity", description="There are no attribute changes to modify the entity.")

    # Attributes tests
    def _test_update_object_with_required_attribute_as_none_should_fail(self, attribute):
        """ Update an object with a required attribute as None
        """
        setattr(self.sdkobject, attribute.local_name, None)
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertErrorEqual(connection.response.errors, title="Invalid input", description="This value is mandatory.", rest_name=attribute.rest_name)

    def _test_update_object_with_attribute_with_choices_as_none_should_fail(self, attribute):
        """ Update an objet with an attribute with choices as none should fail
        """
        setattr(self.sdkobject, attribute.local_name, None)
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertIsNone(getattr(obj, attribute.local_name), "%s should be none but was %s instead" % (attribute.local_name, getattr(obj, attribute.local_name)))

    def _test_update_object_with_attribute_not_in_allowed_choices_list_should_fail(self, attribute):
        """ Update an object with a wrong choice attribute
        """
        setattr(self.sdkobject, attribute.local_name, "A random value")
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 409)
        # self.assertErrorEqual(connection.response.errors, title="Invalid input", description="Invalid input", rest_name=attribute.rest_name)

    def _test_update_object_with_attribute_as_none_should_succeed(self, attribute):
        """ Update an objet with an attribute as none
        """

        setattr(self.sdkobject, attribute.local_name, None)
        (obj, connection) = self.sdkobject.save()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 200)
        self.assertIsNone(getattr(obj, attribute.local_name), "%s should be none but was %s instead" % (attribute.local_name, getattr(obj, attribute.local_name)))


# DELETE TESTS

class DeleteTestMaker(_TestMaker):
    """ TestCase for create objects
    """

    def __init__(self, parent, sdkobject, helper):
        """ Initializes a test case for creating objects
        """
        super(DeleteTestMaker, self).__init__(helper=helper)
        self.parent = parent
        self.sdkobject = sdkobject

        # Object tests
        self.register_test("_test_delete_object_without_authentication_should_fail")
        self.register_test("_test_delete_object_with_valid_id_should_succeed")
        self.register_test("_test_delete_object_with_wrong_id_should_fail")

        # No Attribute tests

    def suite(self):
        """ Inject generated tests
        """
        DeleteTestCase.parent = self.parent
        DeleteTestCase.sdkobject = self.sdkobject
        DeleteTestCase.helper = self.helper

        tests = self.make_tests(sdkobject=self.sdkobject, testcase=DeleteTestCase)
        for test_name, test_func in tests.items():
            setattr(DeleteTestCase, test_name, test_func)

        return TestSuite(list(map(DeleteTestCase, tests)))


class DeleteTestCase(CourgetteTestCase):

    def __init__(self, methodName="runTest"):
        """ Initialize
        """
        CourgetteTestCase.__init__(self, methodName=methodName)
        self.pristine_sdkobject = self.sdkobject.copy()

    def setUp(self):
        """ Setting up create test
        """
        self.last_connection = None
        self.sdkobject = self.pristine_sdkobject.copy()

        self.parent.create_child(self.sdkobject)

    def tearDown(self):
        """ Clean up environment
        """
        if self.sdkobject.id is not None:
            self.sdkobject.delete()

    # Objects tests

    def _test_delete_object_without_authentication_should_fail(self):
        """ Delete an object without authentication
        """
        self.helper.set_api_key(None)
        (obj, connection) = self.sdkobject.delete()
        self.last_connection = connection

        self.helper.set_api_key(self.helper.session.root_object.api_key)

        self.assertConnectionStatus(connection, 401)

    def _test_delete_object_with_valid_id_should_succeed(self):
        """ Delete an object with its id should always succeed with 204 response
        """
        (obj, connection) = self.sdkobject.delete()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 200)
        self.assertEquals(obj.to_dict(), self.sdkobject.to_dict())
        self.sdkobject.id = None  # so it won't be deleted again in tearDown

    def _test_delete_object_with_wrong_id_should_fail(self):
        """ Delete an object with a wrong id should fail with 404 error
        """
        default_id = self.sdkobject.id
        invalid_id = "000-000-000-000-00-000"
        self.sdkobject.id = invalid_id
        (obj, connection) = self.sdkobject.delete()
        self.last_connection = connection

        self.sdkobject.id = default_id

        self.assertConnectionStatus(connection, 404)
        # self.assertErrorEqual(connection.response.errors, title="%s not found" % self.sdkobject.rest_name, description="Cannot find %s with ID %s" % (self.sdkobject.rest_name, invalid_id))


# GET TESTS

class GetTestMaker(_TestMaker):
    """ TestCase for create objects
    """

    def __init__(self, parent, sdkobject, helper):
        """ Initializes a test case for creating objects
        """
        super(GetTestMaker, self).__init__(helper=helper)
        self.parent = parent
        self.sdkobject = sdkobject

        # Object tests
        self.register_test("_test_get_object_without_authentication_should_fail")
        self.register_test("_test_get_object_with_valid_id_should_succeed")
        self.register_test("_test_get_object_with_wrong_id_should_fail")

        # No Attribute tests

    def suite(self):
        """ Inject generated tests
        """
        GetTestCase.parent = self.parent
        GetTestCase.sdkobject = self.sdkobject
        GetTestCase.helper = self.helper

        tests = self.make_tests(sdkobject=self.sdkobject, testcase=GetTestCase)
        for test_name, test_func in tests.items():
            setattr(GetTestCase, test_name, test_func)

        return TestSuite(list(map(GetTestCase, tests)))


class GetTestCase(CourgetteTestCase):

    def __init__(self, methodName="runTest"):
        """ Initialize
        """
        CourgetteTestCase.__init__(self, methodName=methodName)
        self.pristine_sdkobject = self.sdkobject.copy()

    def setUp(self):
        """ Setting up get test
        """
        self.last_connection = None
        self.sdkobject = self.pristine_sdkobject.copy()

        self.parent.create_child(self.sdkobject)

    def tearDown(self):
        """ Clean up environment
        """
        self.sdkobject.delete()

    # Objects tests
    def _test_get_object_without_authentication_should_fail(self):
        """ Get an object without authentication
        """
        self.helper.set_api_key(None)
        (obj, connection) = self.sdkobject.fetch()
        self.last_connection = connection

        self.helper.set_api_key(self.helper.session.root_object.api_key)

        self.assertConnectionStatus(connection, 401)

    def _test_get_object_with_valid_id_should_succeed(self):
        """ Get an object with its id should always succeed with 204 response
        """
        (obj, connection) = self.sdkobject.fetch()
        self.last_connection = connection

        self.assertConnectionStatus(connection, 200)
        self.assertEquals(obj.to_dict(), self.sdkobject.to_dict())

    def _test_get_object_with_wrong_id_should_fail(self):
        """ Get an object with a wrong id should fail with 404 error
        """
        default_id = self.sdkobject.id
        invalid_id = "000-000-000-000-00-000"
        self.sdkobject.id = invalid_id
        (obj, connection) = self.sdkobject.fetch()
        self.last_connection = connection

        self.sdkobject.id = default_id

        self.assertConnectionStatus(connection, 404)
        # self.assertErrorEqual(connection.response.errors, title="%s not found" % self.sdkobject.rest_name, description="Cannot find %s with ID %s" % (self.sdkobject.rest_name, invalid_id))


# GETALL TESTS

class GetAllTestMaker(_TestMaker):
    """ TestCase for create objects

    """
    def __init__(self, parent, sdkobject, helper):
        """ Initializes a test case for creating objects
        """
        super(GetAllTestMaker, self).__init__(helper=helper)
        self.parent = parent
        self.sdkobject = sdkobject

        # Object tests
        self.register_test("_test_get_all_objects_without_authentication_should_fail")
        self.register_test("_test_get_all_objects_without_content_should_success")
        self.register_test("_test_get_all_objects_with_content_should_success")

        # No Attribute tests

    def suite(self):
        """ Inject generated tests
        """
        GetAllTestCase.parent = self.parent
        GetAllTestCase.sdkobject = self.sdkobject
        GetAllTestCase.helper = self.helper

        tests = self.make_tests(sdkobject=self.sdkobject, testcase=GetAllTestCase)
        for test_name, test_func in tests.items():
            setattr(GetAllTestCase, test_name, test_func)

        return TestSuite(list(map(GetAllTestCase, tests)))


class GetAllTestCase(CourgetteTestCase):

    def __init__(self, methodName="runTest"):
        """ Initialize
        """
        CourgetteTestCase.__init__(self, methodName=methodName)
        self.pristine_sdkobject = self.sdkobject.copy()

    def setUp(self):
        """ Setting up get test
        """
        self.last_connection = None
        self.sdkobject = self.pristine_sdkobject.copy()

    def tearDown(self):
        """ Clean up environment
        """
        pass

    # Objects tests
    def _test_get_all_objects_without_authentication_should_fail(self):
        """ Get all object without authentication
        """
        self.helper.set_api_key(None)

        fetcher = self.parent.fetcher_for_rest_name(self.sdkobject.rest_name)
        (fetcher, parent, children) = fetcher.fetch()
        connection = fetcher.current_connection

        self.last_connection = connection

        self.helper.set_api_key(self.helper.session.root_object.api_key)

        self.assertConnectionStatus(connection, 401)

    def _test_get_all_objects_without_content_should_success(self):
        """ Get all object without content should succeed with 200 response
        """
        fetcher = self.parent.fetcher_for_rest_name(self.sdkobject.rest_name)
        (fetcher, parent, children) = fetcher.fetch()
        connection = fetcher.current_connection
        self.last_connection = connection

        self.assertConnectionStatus(connection, 200)

    def _test_get_all_objects_with_content_should_success(self):
        """ Get all object with content should succeed with 200 response
        """
        self.parent.create_child(self.sdkobject)

        fetcher = self.parent.fetcher_for_rest_name(self.sdkobject.rest_name)
        (fetcher, parent, children) = fetcher.fetch()
        connection = fetcher.current_connection

        self.last_connection = connection
        self.sdkobject.delete()

        self.assertConnectionStatus(connection, 200)

    # Attributes tests
    # Filter, Order, Page etc.



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import str
import os
import shutil
from collections import OrderedDict

from monolithe.lib import SDKUtils, TaskManager
from monolithe.generators.lib import TemplateFileWriter


class APIVersionWriter(TemplateFileWriter):
    """ Provide usefull method to write Python files.

    """
    def __init__(self, monolithe_config, api_info):
        """ Initializes a _PythonSDKAPIVersionFileWriter

        """
        super(APIVersionWriter, self).__init__(package="monolithe.generators.lang.python")

        self.api_version = api_info["version"]
        self.api_root = api_info["root"]
        self.api_prefix = api_info["prefix"]

        self.monolithe_config = monolithe_config
        self._output = self.monolithe_config.get_option("output", "transformer")
        self._transformation_name = self.monolithe_config.get_option("name", "transformer")
        self._class_prefix = self.monolithe_config.get_option("class_prefix", "transformer")
        self._product_accronym = self.monolithe_config.get_option("product_accronym")
        self._product_name = self.monolithe_config.get_option("product_name")

        self.output_directory = "%s/python/%s/%s" % (self._output, self._transformation_name, SDKUtils.get_string_version(self.api_version))
        self.override_folder = os.path.normpath("%s/../../__overrides" % self.output_directory)
        self.fetchers_path = "/fetchers/"

        with open("%s/python/__code_header" % self._output, "r") as f:
            self.header_content = f.read()

    def perform(self, specifications):
        """
        """
        self.model_filenames = dict()
        self.fetcher_filenames = dict()

        task_manager = TaskManager()
        for rest_name, specification in specifications.items():
            task_manager.start_task(method=self._write_model, specification=specification, specification_set=specifications)
            task_manager.start_task(method=self._write_fetcher, specification=specification, specification_set=specifications)
        task_manager.wait_until_exit()

        self._write_session()
        self._write_info()
        self._write_init_models(filenames=self.model_filenames)
        self._write_init_fetchers(filenames=self.fetcher_filenames)
        self._write_attrs_defaults()

    def _write_session(self):
        """ Write SDK session file

            Args:
                version (str): the version of the server

        """
        base_name = "%ssession" % self._product_accronym.lower()
        filename = "%s%s.py" % (self._class_prefix.lower(), base_name)
        override_content = self._extract_override_content(base_name)

        self.write(destination=self.output_directory, filename=filename, template_name="session.py.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   override_content=override_content,
                   header=self.header_content)

    def _write_info(self):
        """ Write API Info file
        """
        self.write(destination=self.output_directory, filename="sdkinfo.py", template_name="sdkinfo.py.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   product_name=self._product_name,
                   name=self._transformation_name,
                   header=self.header_content)

    def _write_init_models(self, filenames):
        """ Write init file

            Args:
                filenames (dict): dict of filename and classes

        """

        self.write(destination=self.output_directory, filename="__init__.py", template_name="__init_model__.py.tpl",
                   filenames=self._prepare_filenames(filenames),
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   header=self.header_content)

    def _prepare_filenames(self, filenames, suffix=''):
        """
        """
        formatted_filenames = {}

        for filename, classname in filenames.items():
            formatted_filenames[filename[:-3]] = str("%s%s%s" % (self._class_prefix, classname, suffix))

        return OrderedDict(sorted(formatted_filenames.items()))

    def _write_model(self, specification, specification_set):
        """ Write autogenerate specification file

        """
        filename = "%s%s.py" % (self._class_prefix.lower(), specification.entity_name.lower())

        override_content = self._extract_override_content(specification.entity_name)
        constants = self._extract_constants(specification)
        superclass_name = "NURESTRootObject" if specification.rest_name == self.api_root else "NURESTObject"

        self.write(destination=self.output_directory, filename=filename, template_name="model.py.tpl",
                   specification=specification,
                   specification_set=specification_set,
                   version=self.api_version,
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   override_content=override_content,
                   superclass_name=superclass_name,
                   constants=constants,
                   header=self.header_content)

        self.model_filenames[filename] = specification.entity_name

    def _write_init_fetchers(self, filenames):
        """ Write fetcher init file

            Args:
                filenames (dict): dict of filename and classes

        """
        destination = "%s%s" % (self.output_directory, self.fetchers_path)
        self.write(destination=destination, filename="__init__.py", template_name="__init_fetcher__.py.tpl",
                   filenames=self._prepare_filenames(filenames, suffix='Fetcher'),
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   header=self.header_content)

    def _write_fetcher(self, specification, specification_set):
        """ Write fetcher

        """
        destination = "%s%s" % (self.output_directory, self.fetchers_path)
        base_name = "%s_fetcher" % specification.entity_name_plural.lower()
        filename = "%s%s.py" % (self._class_prefix.lower(), base_name)
        override_content = self._extract_override_content(base_name)

        self.write(destination=destination, filename=filename, template_name="fetcher.py.tpl",
                   specification=specification,
                   specification_set=specification_set,
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   override_content=override_content,
                   header=self.header_content)

        self.fetcher_filenames[filename] = specification.entity_name_plural

    def _write_attrs_defaults(self):
        """
        """
        attrs_defaults_path = "%s/../../__attributes_defaults" % self.output_directory

        if not os.path.exists(attrs_defaults_path):
            return None

        defaults_name = "attrs_defaults.ini"
        specific_defaults_name = "%s_attrs_defaults.ini" % self.api_version
        target_folder = "%s/resources" % self.output_directory

        specific_defaults_path = "%s/%s" % (attrs_defaults_path, specific_defaults_name)
        if os.path.exists(specific_defaults_path):
            return (specific_defaults_path, "%s/%s" % (target_folder, defaults_name), target_folder)

        general_defaults_path = "%s/%s" % (attrs_defaults_path, defaults_name)
        if not os.path.exists(general_defaults_path):
            return

        attributes_file = (general_defaults_path, "%s/%s" % (target_folder, defaults_name), target_folder)

        if not attributes_file:
            return

        src, dst, target = attributes_file

        if not os.path.exists(target):
            os.makedirs(target)

        shutil.copyfile(src, dst)

    def _extract_override_content(self, name):
        """
        """
        # find override file
        specific_override_path = "%s/%s_%s%s.override.py" % (self.override_folder, self.api_version, self._class_prefix.lower(), name.lower())
        generic_override_path = "%s/%s%s.override.py" % (self.override_folder, self._class_prefix.lower(), name.lower())
        final_path = specific_override_path if os.path.exists(specific_override_path) else generic_override_path

        # Read override from file
        override_content = None
        if os.path.isfile(final_path):
            override_content = open(final_path).read()

        return override_content

    def _extract_constants(self, specification):
        """ Removes attributes and computes constants

        """
        constants = {}

        for attribute in specification.attributes:

            if attribute.allowed_choices and len(attribute.allowed_choices) > 0:

                name = attribute.local_name.upper()

                for choice in attribute.allowed_choices:
                    constants["CONST_%s_%s" % (name, choice.upper())] = choice

        return constants



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import object
import importlib

from monolithe.lib.sdkutils import SDKUtils
from bambou import NURESTModelController


class SDKLoader(object):
    """
    """
    def __init__(self, version, sdk_identifier):
        """
        """
        self._sdk_identifier = sdk_identifier
        self._version = SDKUtils.get_string_version(version)
        self._sdk_module = importlib.import_module("%s.%s" % (self._sdk_identifier, self._version))
        self._sdk_utils_module = importlib.import_module("%s.utils" % self._sdk_identifier)

    @property
    def sdk(self):
        return self._sdk_module

    @property
    def sdk_utils(self):
        return self._sdk_utils_module

    @property
    def version(self):
        return self._version

    @property
    def sdk_identifier(self):
        return self._sdk_identifier

    def class_from_rest_name(self, rest_name):
        """
        """
        return NURESTModelController.get_first_model_with_rest_name(rest_name=rest_name)

    def get_instance_from_rest_name(self, rest_name):
        """
        """
        klass = self.class_from_rest_name(rest_name)

        if klass:
            return klass()

        return None



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals

import os
import shutil

from monolithe.lib import Printer
from monolithe.generators.lib import Generator
from monolithe.generators.managers import MainManager, CLIManager, VanillaManager
from .sdkapiversiongenerator import SDKAPIVersionGenerator


class SDKGenerator(Generator):

    def cleanup(self):
        output = self.config.get_option("output", "transformer")
        language = self.config.language

        overrides_path = "%s/%s/__overrides" % (output, language)
        if os.path.exists(overrides_path):
            shutil.rmtree(overrides_path)

        attrs_defaults_path = "%s/%s/__attributes_defaults" % (output, language)
        if os.path.exists(attrs_defaults_path):
            shutil.rmtree(attrs_defaults_path)

        code_header_path = "%s/%s/__code_header" % (output, language)
        if os.path.exists(code_header_path):
            os.remove(code_header_path)

    def generate(self, specification_info):
        user_vanilla = self.config.get_option("user_vanilla", "transformer")
        output = self.config.get_option("output", "transformer")
        name = self.config.get_option("name", "transformer")
        lang = self.config.language

        if not os.path.exists(os.path.join(output, lang)):
            os.makedirs(os.path.join(output, lang))

        vanilla_manager = VanillaManager(monolithe_config=self.config)
        vanilla_manager.execute(output_path="%s/%s" % (output, lang))

        self.install_user_vanilla(user_vanilla_path=user_vanilla, output_path="%s/%s" % (output, lang))

        version_generator = SDKAPIVersionGenerator(self.config)
        apiversions = []

        for info in specification_info:
            Printer.log("transforming specifications into %s for version %s..." % (lang, info["api"]["version"]))
            apiversions.append(info["api"]["version"])

        version_generator.generate(specification_info=specification_info)

        Printer.log("assembling...")
        manager = MainManager(monolithe_config=self.config)
        manager.execute(apiversions=apiversions)

        cli_manager = CLIManager(monolithe_config=self.config)
        cli_manager.execute()

        self.cleanup()
        Printer.success("%s generation complete and available in \"%s/%s\"" % (name, output, self.config.language))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from monolithe.generators.lib import TemplateFileWriter


class CLIWriter(TemplateFileWriter):
    """ Implements `write_cli` that writes CLI python files
    """

    def __init__(self, monolithe_config):
        """
        """
        super(CLIWriter, self).__init__(package="monolithe.generators.lang.python")

        self.monolithe_config = monolithe_config
        self._output = self.monolithe_config.get_option("output", "transformer")
        self._transformation_name = self.monolithe_config.get_option("name", "transformer")
        self._cli_name = self.monolithe_config.get_option("cli_name", "transformer")
        self._class_prefix = self.monolithe_config.get_option("class_prefix", "transformer")
        self._product_accronym = self.monolithe_config.get_option("product_accronym")
        self._product_name = self.monolithe_config.get_option("product_name")
        self.output_directory = "%s/python/%s/cli" % (self._output, self._transformation_name)

        with open("%s/python/__code_header" % self._output, "r") as f:
            self.header_content = f.read()

    def perform(self):
        """
        """
        self._write_init()
        self._write_cli()
        self._write_commands()
        self._write_printer()
        self._write_utils()

    def _write_init(self):
        """
        """
        self.write(destination=self.output_directory, filename="__init__.py", template_name="cli__init__.py.tpl",
                   header=self.header_content)

    def _write_cli(self):
        """
        """
        self.write(destination=self.output_directory, filename="cli.py", template_name="cli.py.tpl",
                   product_accronym=self._product_accronym,
                   product_name=self._product_name,
                   header=self.header_content)

    def _write_commands(self):
        """
        """
        self.write(destination=self.output_directory, filename="commands.py", template_name="cli_commands.py.tpl",
                   product_accronym=self._product_accronym,
                   product_name=self._product_name,
                   cli_name=self._cli_name,
                   header=self.header_content)

    def _write_printer(self):
        """
        """
        self.write(destination=self.output_directory, filename="printer.py", template_name="cli_printer.py.tpl",
                   header=self.header_content)

    def _write_utils(self):
        """
        """
        self.write(destination=self.output_directory, filename="utils.py", template_name="cli_utils.py.tpl",
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   name=self._transformation_name,
                   header=self.header_content)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from pkg_resources import iter_entry_points, load_entry_point

_language_plugin_cache = {}


def load_language_plugins(language, key):
    """
    """
    global _language_plugin_cache

    if key in _language_plugin_cache:
        return True, _language_plugin_cache[language][key]

    method = None
    group = 'monolithe.plugin.lang.%s' % language
    for entry_point in iter_entry_points(group=group, name='info'):
        method = load_entry_point(entry_point.dist, group=group, name='info')
        break

    if method:
        _language_plugin_cache[language] = method()
        return True, _language_plugin_cache[language][key]
    else:
        _language_plugin_cache[language] = {'APIVersionWriter': None, 'PackageWriter': None, 'CLIWriter': None, 'VanillaWriter': None, 'get_idiomatic_name': None, 'get_type_name': None}
        return False, None


def apply_extension(extension, specification):

    if "model" in extension:
        if "model" not in specification:
            specification["model"] = {}

        for key, value in extension["model"].items():
            if value is not None:
                if key not in specification["model"] or specification["model"][key] is None:
                    specification["model"][key] = value

    if "attributes" in extension:
        if "attributes" not in specification:
            specification["attributes"] = []

        for attr in extension["attributes"]:
            attribute_name = attr["name"]

            found = False
            for a in specification["attributes"]:
                if a["name"] == attribute_name:
                    found = True
                    break

            if not found:
                specification["attributes"].append(attr)

    if "children" in extension:
        if "children" not in specification:
            specification["children"] = []

        for api in extension["children"]:
            api_name = api["rest_name"]

            found = False
            for a in specification["children"]:
                if a["rest_name"] == api_name:
                    found = True
                    break

            if not found:
                specification["children"].append(api)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import object


class SpecificationAPI(object):
    """ Describe an object API

    """
    def __init__(self, specification=None, data=None):
        """
        """
        self.specification = specification

        self.rest_name = ""
        self.allows_get = False
        self.allows_create = False
        self.allows_update = False
        self.allows_delete = False
        self.allows_bulk_create = False
        self.allows_bulk_update = False
        self.allows_bulk_delete = False
        self.deprecated = False
        self.relationship = "child"

        if data:
            self.from_dict(data)

    def from_dict(self, data):
        """

        """
        # mandatory
        self.rest_name = data["rest_name"]

        self.allows_get = data["get"] if "get" in data else False
        self.allows_create = data["create"] if "create" in data else False
        self.allows_update = data["update"] if "update" in data else False
        self.allows_delete = data["delete"] if "delete" in data else False
        self.allows_bulk_create = data["bulk_create"] if "bulk_create" in data else False
        self.allows_bulk_update = data["bulk_update"] if "bulk_update" in data else False
        self.allows_bulk_delete = data["bulk_delete"] if "bulk_delete" in data else False
        self.deprecated = data["deprecated"] if "deprecated" in data else False
        self.relationship = data["relationship"] if "relationship" in data else "child"

    def to_dict(self):
        """
        """

        data = {}

        data["rest_name"] = self.rest_name
        data["get"] = self.allows_get
        data["create"] = self.allows_create
        data["update"] = self.allows_update
        data["delete"] = self.allows_delete
        data["bulk_create"] = self.allows_bulk_create
        data["bulk_update"] = self.allows_bulk_update
        data["bulk_delete"] = self.allows_bulk_delete
        data["deprecated"] = self.deprecated
        data["relationship"] = self.relationship

        return data



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from future import standard_library
standard_library.install_aliases()

import os
from configparser import RawConfigParser

from monolithe.lib import TaskManager
from monolithe.generators.lib import TemplateFileWriter


class APIVersionWriter(TemplateFileWriter):
    """ Provide usefull method to write Go files.

    """
    def __init__(self, monolithe_config, api_info):
        """ Initializes a _GoSDKAPIVersionFileWriter

        """
        super(APIVersionWriter, self).__init__(package="monolithe.generators.lang.go")

        self.monolithe_config = monolithe_config

        self.api_version = api_info["version"]
        self.api_root = api_info["root"]
        self.api_prefix = api_info["prefix"]

        self._output = self.monolithe_config.get_option("output", "transformer")
        self._transformation_name = self.monolithe_config.get_option("name", "transformer")
        self._product_accronym = self.monolithe_config.get_option("product_accronym")
        self._product_name = self.monolithe_config.get_option("product_name")

        self.output_directory = "%s/go/%s" % (self._output, self._transformation_name)

        self.attrs_defaults = RawConfigParser()
        path = "%s/go/__attributes_defaults/attrs_defaults.ini" % self._output
        self.attrs_defaults.optionxform = str
        self.attrs_defaults.read(path)

        with open("%s/go/__code_header" % self._output, "r") as f:
            self.header_content = f.read()

    def perform(self, specifications):
        """
        """
        self._write_info()
        self._write_session()

        task_manager = TaskManager()
        for rest_name, specification in specifications.items():
            task_manager.start_task(method=self._write_model, specification=specification, specification_set=specifications)
        task_manager.wait_until_exit()

        self._format()

    def _write_info(self):
        """
        """
        self.write(destination=self.output_directory, filename="sdkinfo.go", template_name="sdkinfo.go.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   product_name=self._product_name,
                   name=self._transformation_name,
                   header=self.header_content)

    def _write_session(self):
        """
        """
        self.write(destination=self.output_directory, filename="session.go", template_name="session.go.tpl",
                   version=self.api_version,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   name=self._transformation_name,
                   header=self.header_content)

    def _write_model(self, specification, specification_set):
        """
        """
        filename = "%s.go" % (specification.entity_name.lower())

        defaults = {}
        section = specification.entity_name
        if self.attrs_defaults.has_section(section):
            for attribute in self.attrs_defaults.options(section):
                defaults[attribute] = self.attrs_defaults.get(section, attribute)

        self.write(destination=self.output_directory, filename=filename, template_name="model.go.tpl",
                   specification=specification,
                   specification_set=specification_set,
                   name=self._transformation_name,
                   header=self.header_content,
                   attribute_defaults=defaults)

        return (filename, specification.entity_name)

    def _format(self):
        """
        """
        os.system("gofmt -w '%s' >/dev/null 2>&1" % self.output_directory)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals

from builtins import object

from monolithe.generators.managers import APIVersionManager, DocumentationManager


class SDKAPIVersionGenerator(object):
    """ Generate SDK

    """
    def __init__(self, monolithe_config):
        """
        """
        self.monolithe_config = monolithe_config

    def generate(self, specification_info):
        """
        """
        for info in specification_info:
            manager = APIVersionManager(monolithe_config=self.monolithe_config)
            manager.execute(specifications=info["specifications"], api_info=info["api"])
            doc_manager = DocumentationManager(monolithe_config=self.monolithe_config)
            doc_manager.execute(info["specifications"], info["api"])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re


def _string_clean(string):
    """
    """
    rep = {
        "IPID": "IpID",
        "VCenter": "Vcenter",
        "vCenter": "Vcenter",
        "VPort": "Vport",
        "IPv6": "Ipv6",
        "IPv4": "Ipv4",
        "DHCPv4": "Dhcpv4",
        "DHCPv6": "Dhcpv6"
    }

    rep = dict((re.escape(k), v) for k, v in rep.items())
    pattern = re.compile("|".join(list(rep.keys())))

    return pattern.sub(lambda m: rep[re.escape(m.group(0))], string)


def get_idiomatic_name(name):
    """
    """
    first_cap_re = re.compile("(.)([A-Z](?!s([A-Z])*)[a-z]+)")
    all_cap_re = re.compile("([a-z0-9])([A-Z])")

    s1 = first_cap_re.sub(r"\1_\2", _string_clean(name))

    return all_cap_re.sub(r"\1_\2", s1).lower()


def get_type_name(type_name, sub_type=None):
    """
    """
    if type_name in ("string", "enum"):
        return "str"

    if type_name == "boolean":
        return "bool"

    if type_name in ("integer", "long"):
        return "int"

    if type_name == "time":
        return "float"

    if type_name == "object":
        return "dict"

    return type_name



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals

from builtins import object

from monolithe.lib import SDKUtils
from .specification_api import SpecificationAPI
from .specification_attribute import SpecificationAttribute


class Specification(object):
    """ Defines a specification object

    """

    def __init__(self, filename, monolithe_config=None, data=None):
        """ Initializes a model object

            Example:
                name: EnterpriseNetwork
                instance_name: enterprise_network
                entity_name_plural: EnterpriseNetworks
                instance_name_plural: enterprise_networks
                rest_name: enterprisenetwork
                resource_name: enterprisenetworks
                package: network
        """
        self.monolithe_config = monolithe_config
        self.filename = filename

        self.allows_create = False
        self.allows_delete = False
        self.allows_get = False
        self.allows_update = False
        self.description = None
        self.entity_name_plural = None  # the original name in plural
        self.extends = []
        self.instance_name = None  # Name of the object as an instance
        self.instance_name_plural = None  # Name of the object as an instance of array or fetcher
        self.is_root = False
        self.package = None
        self.resource_name = None  # The name of the resource used in URI
        self.rest_name = None  # The remote name of the object
        self._entity_name = None  # The original name of the object
        self.userlabel = None
        self.template = False
        self.allowed_job_commands = None

        self.attributes = []
        self.child_apis = []
        self.parent_apis = []

        if data:
            self.from_dict(data=data)

    @property
    def entity_name(self):
        """
        """
        return self._entity_name

    @entity_name.setter
    def entity_name(self, value):
        """
        """
        self._entity_name = value

        language = self.monolithe_config.language if self.monolithe_config else 'python'

        if value:
            self.instance_name = SDKUtils.get_idiomatic_name_in_language(name=value, language=language)
            self.entity_name_plural = SDKUtils.get_plural(singular_name=value)
            self.instance_name_plural = SDKUtils.get_idiomatic_name_in_language(name=self.entity_name_plural, language=language)

    def to_dict(self):
        """ Transform the current specification to a dictionary

        """

        data = {"model": {}}

        data["model"]["description"] = self.description
        data["model"]["entity_name"] = self.entity_name
        data["model"]["package"] = self.package
        data["model"]["resource_name"] = self.resource_name
        data["model"]["rest_name"] = self.rest_name
        data["model"]["extends"] = self.extends
        data["model"]["get"] = self.allows_get
        data["model"]["update"] = self.allows_update
        data["model"]["create"] = self.allows_create
        data["model"]["delete"] = self.allows_delete
        data["model"]["root"] = self.is_root
        data["model"]["userlabel"] = self.userlabel
        data["model"]["template"] = self.template
        data["model"]["allowed_job_commands"] = self.allowed_job_commands

        data["attributes"] = []
        for attribute in self.attributes:
            data["attributes"].append(attribute.to_dict())

        data["children"] = []
        for api in self.child_apis:
            data["children"].append(api.to_dict())

        return data

    def from_dict(self, data):
        """ Fill the current object with information from the specification

        """
        if "model" in data:
            model = data["model"]
            self.description = model["description"] if "description" in model else None
            self.package = model["package"] if "package" in model else None
            self.extends = model["extends"] if "extends" in model else []
            self.entity_name = model["entity_name"] if "entity_name" in model else None
            self.rest_name = model["rest_name"] if "rest_name" in model else None
            self.resource_name = model["resource_name"] if "resource_name" in model else None
            self.allows_get = model["get"] if "get" in model else False
            self.allows_create = model["create"] if "create" in model else False
            self.allows_update = model["update"] if "update" in model else False
            self.allows_delete = model["delete"] if "delete" in model else False
            self.is_root = model["root"] if "root" in model else False
            self.userlabel = model["userlabel"] if "userlabel" in model else None
            self.template = model["template"] if "template" in model else False
            self.allowed_job_commands = model["allowed_job_commands"] if "allowed_job_commands" in model else None

        if "attributes" in data:
            self.attributes = self._get_attributes(data["attributes"])

        if "children" in data:
            self.child_apis = self._get_apis(data["children"])

    def _get_apis(self, apis):
        """ Process apis for the given model

            Args:
                model: the model processed
                apis: the list of apis availble for the current model
                relations: dict containing all relations between resources
        """
        ret = []

        for data in apis:
            ret.append(SpecificationAPI(specification=self, data=data))

        return sorted(ret, key=lambda x: x.rest_name[1:])

    def _get_attributes(self, attributes):
        """

        """
        ret = []

        for data in attributes:
            ret.append(SpecificationAttribute(specification=self, data=data))

        return sorted(ret, key=lambda x: x.name[1:])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
from configparser import RawConfigParser
from monolithe.lib import SDKUtils, TaskManager
from monolithe.generators.lib import TemplateFileWriter

from future import standard_library
from urlparse import urlparse
standard_library.install_aliases()


class APIVersionWriter(TemplateFileWriter):
    """ Provide useful method to write Java files.

    """
    def __init__(self, monolithe_config, api_info):
        """ Initializes a _JavaSDKAPIVersionFileWriter

        """
        super(APIVersionWriter, self).__init__(package="monolithe.generators.lang.java")

        self.api_version = api_info["version"]
        self._api_version_string = SDKUtils.get_string_version(self.api_version)
        self.api_root = api_info["root"]
        self.api_prefix = api_info["prefix"]

        self.monolithe_config = monolithe_config
        self._output = self.monolithe_config.get_option("output", "transformer")
        self._name = self.monolithe_config.get_option("name", "transformer")
        self._class_prefix = ""
        self._product_accronym = self.monolithe_config.get_option("product_accronym")
        self._product_name = self.monolithe_config.get_option("product_name")
        self._url = self.monolithe_config.get_option("url", "transformer")

        self._package_prefix = self._get_package_prefix(self._url)
        self._package_name = self._package_prefix + '.' + self._name + '.' + SDKUtils.get_string_version(self.api_version)
        self._package_subdir = self._package_name.replace('.', '/')

        self._base_output_directory = "%s/java" % (self._output)
        self.output_directory = "%s/src/main/java/%s" % (self._base_output_directory, self._package_subdir)
        self.override_folder = os.path.normpath("%s/__overrides" % self._base_output_directory)
        self.fetchers_path = "/fetchers/"

        self.attrs_defaults = RawConfigParser()
        path = "%s/java/__attributes_defaults/attrs_defaults.ini" % self._output
        self.attrs_defaults.optionxform = str
        self.attrs_defaults.read(path)

        self.attrs_types = RawConfigParser()
        path = "%s/java/__attributes_defaults/attrs_types.ini" % self._output
        self.attrs_types.optionxform = str
        self.attrs_types.read(path)

        self.library_version = self.monolithe_config.get_option("version", "transformer")

        with open("%s/java/__code_header" % self._output, "r") as f:
            self.header_content = f.read()

    def perform(self, specifications):
        """
        """
        self._set_enum_list_local_type(specifications)
        self._write_info()
        self._write_session()
        self._write_build_file()

        task_manager = TaskManager()
        for rest_name, specification in specifications.items():
            task_manager.start_task(method=self._write_model, specification=specification, specification_set=specifications)
            task_manager.start_task(method=self._write_fetcher, specification=specification, specification_set=specifications)
        task_manager.wait_until_exit()

    def _write_session(self):
        """ Write SDK session file

            Args:
                version (str): the version of the server

        """
        base_name = "%sSession" % self._product_accronym
        filename = "%s%s.java" % (self._class_prefix, base_name)
        override_content = self._extract_override_content(base_name)

        self.write(destination=self.output_directory,
                   filename=filename,
                   template_name="session.java.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   root_api=self.api_root,
                   name=self._name,
                   api_prefix=self.api_prefix,
                   override_content=override_content,
                   header=self.header_content,
                   version_string=self._api_version_string,
                   package_name=self._package_name)

    def _write_info(self):
        """ Write API Info file
        """
        self.write(destination=self.output_directory,
                   filename="SdkInfo.java",
                   template_name="sdkinfo.java.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   product_name=self._product_name,
                   name=self._name,
                   header=self.header_content,
                   version_string=self._api_version_string,
                   package_name=self._package_name)

    def _write_model(self, specification, specification_set):
        """ Write autogenerate specification file

        """
        filename = "%s%s.java" % (self._class_prefix, specification.entity_name)

        override_content = self._extract_override_content(specification.entity_name)
        superclass_name = "RestRootObject" if specification.rest_name == self.api_root else "RestObject"

        defaults = {}
        section = specification.entity_name
        if self.attrs_defaults.has_section(section):
            for attribute in self.attrs_defaults.options(section):
                defaults[attribute] = self.attrs_defaults.get(section, attribute)

        self.write(destination=self.output_directory,
                   filename=filename,
                   template_name="model.java.tpl",
                   specification=specification,
                   specification_set=specification_set,
                   version=self.api_version,
                   name=self._name,
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   override_content=override_content,
                   superclass_name=superclass_name,
                   header=self.header_content,
                   version_string=self._api_version_string,
                   package_name=self._package_name,
                   attribute_defaults=defaults)

        return (filename, specification.entity_name)

    def _write_fetcher(self, specification, specification_set):
        """ Write fetcher

        """
        destination = "%s%s" % (self.output_directory, self.fetchers_path)
        base_name = "%sFetcher" % specification.entity_name_plural
        filename = "%s%s.java" % (self._class_prefix, base_name)
        override_content = self._extract_override_content(base_name)

        self.write(destination=destination,
                   filename=filename,
                   template_name="fetcher.java.tpl",
                   specification=specification,
                   specification_set=specification_set,
                   class_prefix=self._class_prefix,
                   product_accronym=self._product_accronym,
                   override_content=override_content,
                   header=self.header_content,
                   name=self._name,
                   version_string=self._api_version_string,
                   package_name=self._package_name)

        return (filename, specification.entity_name_plural)

    def _write_build_file(self):
        """ Write Maven build file (pom.xml)

        """
        self.write(destination=self._base_output_directory,
                   filename="pom.xml",
                   template_name="pom.xml.tpl",
                   version=self.api_version,
                   product_accronym=self._product_accronym,
                   class_prefix=self._class_prefix,
                   root_api=self.api_root,
                   api_prefix=self.api_prefix,
                   product_name=self._product_name,
                   name=self._name,
                   header=self.header_content,
                   version_string=self._api_version_string,
                   package_prefix=self._package_prefix,
                   library_version=self.library_version)

    def _extract_override_content(self, name):
        """
        """
        # find override file
        specific_override_path = "%s/%s_%s%s.override.java" % (self.override_folder, self.api_version, self._class_prefix, name.title())
        generic_override_path = "%s/%s%s.override.java" % (self.override_folder, self._class_prefix, name.title())
        final_path = specific_override_path if os.path.exists(specific_override_path) else generic_override_path

        # Read override from file
        override_content = None
        if os.path.isfile(final_path):
            override_content = open(final_path).read()

        return override_content

    def _get_package_prefix(self, url):
        """
        """
        hostname_parts = self._get_hostname_parts(url)

        package_name = ""
        for index, hostname_part in enumerate(reversed(hostname_parts)):
            package_name = package_name + hostname_part
            if index < len(hostname_parts) - 1:
                package_name = package_name + '.'

        return package_name

    def _get_hostname_parts(self, url):
        """
        """
        if url.find("http://") != 0:
            url = "http://" + url

        hostname = urlparse(url).hostname
        hostname_parts = hostname.split('.')

        valid_hostname_parts = []
        for hostname_part in hostname_parts:
            if hostname_part != "www":
                valid_hostname_parts.append(hostname_part)

        return valid_hostname_parts

    def _set_enum_list_local_type(self, specifications):
        """ This method is needed until get_type_name() is enhanced to include specification subtype and local_name
        """
        for rest_name, specification in specifications.items():
            for attribute in specification.attributes:
                if attribute.type == "enum":
                    enum_type = attribute.local_name[0:1].upper() + attribute.local_name[1:]
                    attribute.local_type = enum_type
                elif attribute.type == "object":
                    attr_type = "Object"
                    if self.attrs_types.has_option(specification.entity_name, attribute.local_name):
                        type = self.attrs_types.get(specification.entity_name, attribute.local_name)
                        if type:
                            attr_type = type
                    attribute.local_type = attr_type
                elif attribute.type == "list":
                    if attribute.subtype == "enum":
                        enum_subtype = attribute.local_name[0:1].upper() + attribute.local_name[1:]
                        attribute.local_type = "java.util.List<" + enum_subtype + ">"
                    elif attribute.subtype == "object":
                        attr_subtype = "com.fasterxml.jackson.databind.JsonNode"
                        if self.attrs_types.has_option(specification.entity_name, attribute.local_name):
                            subtype = self.attrs_types.get(specification.entity_name, attribute.local_name)
                            if subtype:
                                attr_subtype = subtype
                        attribute.local_type = "java.util.List<" + attr_subtype + ">"
                    elif attribute.subtype == "entity":
                        attribute.local_type = "java.util.List<com.fasterxml.jackson.databind.JsonNode>"
                    elif attribute.subtype == "string":
                        attribute.local_type = "java.util.List<String>"
                    else:
                        attribute.local_type = "java.util.List<"+attribute.subtype+">"



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import object

import os
import shutil

from monolithe.lib import Printer
from monolithe.specifications import SpecificationAPI


class Generator(object):

    def __init__(self, directory_manager, branches):
        self.directory_manager = directory_manager
        self.branches = branches

    @property
    def config(self):
        return self.directory_manager.monolithe_config

    def run(self, branches=None):
        specification_info = []

        if branches:
            for branch in branches:
                Printer.log("Switching to branch %s" % branch)
                self.directory_manager.switch_branch(branch)
                specification_info.append(self.get_specifications_infos_from_folder())
        else:
            specification_info.append(self.get_specifications_infos_from_folder())

        self.generate(specification_info=specification_info)

    def get_specifications_infos_from_folder(self):
        Printer.log("retrieving specifications from folder \"%s\"" % (self.directory_manager.folder))
        api_info = self.directory_manager.get_api_info()
        specifications = self.directory_manager.get_all_specifications()
        self._resolve_parent_apis(specifications)
        Printer.log("%d specifications retrieved from folder \"%s\" (api version: %s)" % (len(specifications), self.directory_manager.folder, api_info["version"]))
        return {"specifications": specifications, "api": api_info}

    def generate(self, specification_info):
        raise NotImplementedError()

    def install_user_vanilla(self, user_vanilla_path, output_path, multi_lang=True):
        """
        """
        if not user_vanilla_path or not len(user_vanilla_path):
            return

        if multi_lang:
            user_vanilla_path = os.path.join(user_vanilla_path, self.config.language)
        else:
            user_vanilla_path = os.path.join(user_vanilla_path)

        if not os.path.exists(user_vanilla_path):
            Printer.warn("Could not find user vanilla folder at path %s. Ignoring" % user_vanilla_path)
            return

        for item in os.listdir(user_vanilla_path):
            s = os.path.join(user_vanilla_path, item)
            d = os.path.join(output_path, item)
            if os.path.isdir(s):
                shutil.copytree(s, d, False, None)
            else:
                shutil.copy2(s, d)

    # Utilities
    def _resolve_parent_apis(self, specifications):
        """
        """

        # certainly not the best algo ever... but I need to get somthing done :)
        for specification_rest_name, specification in specifications.items():

            for rest_name, remote_spec in specifications.items():

                for related_child_api in remote_spec.child_apis:

                    if related_child_api.rest_name == specification.rest_name:

                        parent_api = SpecificationAPI(specification=specification)
                        parent_api.rest_name = related_child_api.rest_name
                        if specification.allows_get:
                            parent_api.allows_get = True
                        if specification.allows_create:
                            parent_api.allows_create = True
                        if specification.allows_update:
                            parent_api.allows_update = True
                        if specification.allows_delete:
                            parent_api.allows_Delete = True

                        specification.parent_apis.append(parent_api)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from __future__ import unicode_literals
from builtins import object
import os

from jinja2 import Environment, PackageLoader


class _FileWriter(object):
    """ Writer a file content

    """
    def write(self, destination, filename, content):
        """ Write a file at the specific destination with the content.

            Args:
                destination (string): the destination location
                filename (string): the filename that will be written
                content (string): the content of the filename

        """
        if not os.path.exists(destination):
            try:
                os.makedirs(destination)
            except:  # The directory can be created while creating it.
                pass

        filepath = "%s/%s" % (destination, filename)

        f = open(filepath, "w+")
        f.write(content)
        f.close()


class TemplateFileWriter(_FileWriter):
    """ Write a template file

    """

    def __init__(self, package):
        """ Initializes a FileWriter

        """
        super(TemplateFileWriter, self).__init__()

        templates = "templates"

        self.env = Environment(loader=PackageLoader(package, templates), extensions=["jinja2.ext.do"])

    def write(self, destination, filename, template_name, **kwargs):
        """ Write a file according to the template name

            Args:
                destination (string): the destination location
                filename (string): the filename that will be written
                template_name (string): the name of the template
                kwargs (dict): all attribute that will be passed to the template
        """
        template = self.env.get_template(template_name)
        content = template.render(kwargs)
        super(TemplateFileWriter, self).write(destination=destination, filename=filename, content=content)



__all__ = ['APIVersionWriter', 'plugin_info']

from .writers.apiversionwriter import APIVersionWriter


def plugin_info():
    """ Entry point of your plugin. This will be called by Monolithe to check if
        it should use this plugin to generate a particular language.
    """

    return {
        # 'VanillaWriter' is used to copy some vanilla files.
        'VanillaWriter': None,

        # 'APIVersionWriter' is reponsible to write files for a particular api version.
        'APIVersionWriter': APIVersionWriter,

        # 'PackageWriter' is reponsible to assemble all the api version sets created by APIVersionWriter, if needed
        'PackageWriter': None,

        # 'CLIWriter' is reponsible to write a CLI if needed
        'CLIWriter': None,

        # 'get_idiomatic_name' is a function that will be called when a word from the specification
        # might need to get adjusted to respect the idioms of your language. for instance 'HelloWorld' in Python
        # should be translated to 'hello_world'
        'get_idiomatic_name': None,

        # 'get_type_name' is needed to translate a type from a specification to the name of the type used
        # by your language. For instance, a spec type 'list' needs to be translated to '[]<subtype>' in Go.
        'get_type_name': None
    }



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import object

from monolithe.courgette.lib import CourgetteTestsRunner
from .result import CourgetteResult


class Courgette(object):
    """ Validates API by launching specification-generated tests

    """
    def __init__(self, url, username, password, enterprise, apiversion, sdk_identifier, monolithe_config):
        """ Initializes Courgette

            Args:
                url (string): the url of the server with its port
                username (string): the username to launch tests
                password (string): the password to connect to the server
                enterprise (string): the name of the enterprise to connect to the server
                apiversion (float): the version of the API to connect
                sdk (string): the full name of the SDK to use
        """
        self.url = url
        self.username = username
        self.password = password
        self.enterprise = enterprise
        self.apiversion = apiversion
        self.monolithe_config = monolithe_config
        self.sdk_identifier = sdk_identifier

    def run(self, configurations):
        """ Run all tests

            Returns:
                A dictionnary containing tests results.

        """
        result = CourgetteResult()

        for configuration in configurations:

            runner = CourgetteTestsRunner(url=self.url,
                                          username=self.username,
                                          password=self.password,
                                          enterprise=self.enterprise,
                                          version=self.apiversion,
                                          specification=configuration.specification,
                                          sdk_identifier=self.sdk_identifier,
                                          monolithe_config=self.monolithe_config,
                                          parent_resource=configuration.parent_resource_name,
                                          parent_id=configuration.parent_id,
                                          default_values=configuration.default_values)

            result.add_report(configuration.specification.rest_name + ".spec", runner.run())

        return result



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015, Alcatel-Lucent Inc
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the copyright holder nor the names of its contributors
#       may be used to endorse or promote products derived from this software without
#       specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from builtins import object


class CourgetteResult(object):

    def __init__(self):
        """
            Initializes the CourgetteResult
        """
        self._failures = 0
        self._errors = 0
        self._success = 0
        self._total = 0
        self._reports = {}

    # properties

    @property
    def reports(self):
        return self._reports

    @property
    def failures(self):
        return self._failures

    @property
    def errors(self):
        return self._errors

    @property
    def success(self):
        return self._success

    @property
    def total(self):
        return self._total

    # report management

    def add_report(self, specification_name, report):
        """
            Adds a given report with the given specification_name as key
            to the reports list and computes the number of success, failures
            and errors

            Args:
                specification_name: string representing the specification (with ".spec")
                report: The
        """
        self._reports[specification_name] = report

        self._total = self._total + report.testsRun
        self._failures = self._failures + len(report.failures)
        self._errors = self._errors + len(report.errors)
        self._success = self._total - self._failures - self._errors
